"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.QueryBuilder = void 0;
// Crea los querys
let QueryBuilder = exports.QueryBuilder = void 0;
(function (_QueryBuilder) {
  function query(tableName, options) {
    let sqlParts = ['SELECT', options.columns ?? "*", 'FROM', tableName];
    if (options.where) {
      sqlParts.push("WHERE");
      sqlParts.push(options.where.clause);
    }
    if (options.order) {
      sqlParts.push("ORDER BY");
      sqlParts.push(options.order);
    }
    if (options.limit) {
      sqlParts.push("LIMIT");
      sqlParts.push(options.limit);
      if (options.page) {
        sqlParts.push("OFFSET");
        sqlParts.push(options.limit * (options.page - 1));
      }
    }
    return sqlParts.filter(p => p !== '').join(' ');
  }
  _QueryBuilder.query = query;
  function create(tableName, object) {
    const keys = Object.keys(object);
    const columns = keys.join(', ');
    const values = keys.map(() => '?').join(', ');
    return `INSERT INTO ${tableName} (${columns}) VALUES (${values});`;
  }
  _QueryBuilder.create = create;
  function update(tableName, object, whereClause) {
    // Extrae el valor de "id" y crea un nuevo objeto sin ese dato
    //const { id, ...props } = object
    const values = Object.keys(object).map(k => `${k} = ?`).join(', ');
    return `UPDATE ${tableName} SET ${values} WHERE ${whereClause};`;
  }
  _QueryBuilder.update = update;
  function destroy(tableName, whereClause) {
    return `DELETE FROM ${tableName} WHERE ${whereClause};`;
  }
  _QueryBuilder.destroy = destroy;
  function destroyAll(tableName) {
    return `DELETE FROM ${tableName};`;
  }
  _QueryBuilder.destroyAll = destroyAll;
})(QueryBuilder || (exports.QueryBuilder = QueryBuilder = {}));
var _default = exports.default = QueryBuilder;
//# sourceMappingURL=QueryBuilder.js.map