export type QueryOptions = {
  columns?: string
  page?: number
  limit?: number
  where?: Where,
  order?: string
}

export type Where = {
  clause: string,
  args?: any[],
}

// Crea los querys
export module QueryBuilder {

  export function query(
    tableName: string, 
    options: QueryOptions
  ): string {

    let sqlParts: any[] = [
      'SELECT',
      options.columns ?? "*",
      'FROM',
      tableName,
    ]

    if (options.where) {
      sqlParts.push("WHERE")
      sqlParts.push(options.where.clause)
    }
    
    if (options.order) {
      sqlParts.push("ORDER BY")
      sqlParts.push(options.order)
    }

    if (options.limit) {
      sqlParts.push("LIMIT")
      sqlParts.push(options.limit)

      if (options.page) {
        sqlParts.push("OFFSET")
        sqlParts.push(options.limit * (options.page - 1))
      }
    }

    return sqlParts.filter(p => p !== '').join(' ')
  }

  // Creates the "INSERT" sql statement
  export function create(tableName: string,object: any): string {
    const keys = Object.keys(object)
    const columns = keys.join(', ')
    const values = keys.map(() => '?').join(', ')

    return `INSERT INTO ${tableName} (${columns}) VALUES (${values});`
  }

  // Creates the "Update" sql statement
  export function update(tableName: string, object: any, whereClause: string): string {
    // Extrae el valor de "id" y crea un nuevo objeto sin ese dato
    //const { id, ...props } = object
    const values = Object.keys(object)
      .map(k => `${k} = ?`)
      .join(', ')

    return `UPDATE ${tableName} SET ${values} WHERE ${whereClause};`
  }

  // Creates the "DELETE" sql statement
  export function destroy(tableName: string, whereClause: string): string {
    return `DELETE FROM ${tableName} WHERE ${whereClause};`
  }
  
  // Creates the "DELETE ALL" sql statement
  export function destroyAll(tableName: string): string {
    return `DELETE FROM ${tableName};`
  }
  
}

export default QueryBuilder